/****
 * CSymWndGrid.c
 *
 *	Symbol window methods for the SignWriter key editor.
 *
 *  Copyright 1995 The Center for Sutton Movement Writing. All rights reserved.
 *
 ****/

 
/**
 *
 *  This class subclasses TCL's CGridSelector class to implement
 *  the key editor's symbol window.  It uses a hacked version of
 *  CScrollPane to display a "current symcode" box next to the
 *  scroll bar.  It also overrides CGridSelector's DrawGrid
 *  method to get what we need.
 *
 **/



#include "CSymWndGrid.h"
#include "CApplication.h"
#include "CBureaucrat.h"
#include "CSelector.h"
#include "CSymWindow.h"
#include "CWindow.h"
#include "Global.h"
#include "KeyEditConsts.h"
#include "Symbols.h"
#include "TBUtilities.h"
#include <BDC.h>





/****************************  Initialization  ****************************/


void CSymWndGrid::ISymWndGrid (CView *anEnclosure, CBureaucrat *aSupervisor,
							     short aWidth, short aHeight,
								 short aHEncl, short aVEncl,
								 SizingOption aHSizing, SizingOption aVSizing)
{	
	inherited::IGridSelector(anEnclosure, aSupervisor,
				aHEncl, aVEncl, aHSizing, aVSizing,
				SELnone, 0, SGRID_ROWS, SGRID_COLS,
				SCELL_WIDTH, SCELL_HEIGHT);
}





 /******************************  Grid mapping  *******************************/

 
/***
 * ValidItem
 *
 *  Return TRUE if the specified item
 *  is a defined symbol cell.
 *
 ***/
 
Boolean CSymWndGrid::ValidItem (short theItem)

{		
 	return theItem > SGRID_COLS;			/* ignore label cells */	
}



/***
 * ItemToSymCode
 *
 *	Convert a file grid item number to the equivalent
 *  SignWriter symbol code value.  Translate the grid
 *  class row-major form to the symbol code's column-
 *  major form.
 *
 ***/
 
 short CSymWndGrid::ItemToSymCode (short item)
 {
 	short	row, col;
 	
 	item--;
 	row = (item / SGRID_COLS) - 1;  /* label row adjust */
 	col = item % SGRID_COLS;
 	return (col * (SGRID_ROWS - 1)) + row;
 }
 
 
 
/***
 * SymCodeToItem
 *
 *	Convert a SignWriter symbol code value to 
 *  its equivalent item number in the file grid. 
 *
 ***/
 
 short CSymWndGrid::SymCodeToItem (short symCode)
 {
 	short	row, col;
 	
 	row = symCode % (SGRID_ROWS - 1);
 	col = symCode / (SGRID_ROWS - 1);
 	return ((row + 1) * SGRID_COLS) + col + 1;
 }



 	
 /********************************  Drawing  ********************************/
	

/***
 * DrawGrid (override default method)
 *
 *  Draw grid lines around the items.  Leave top row ungridded;
 *  instead, draw grid label values across it.  
 *
 ***/

void CSymWndGrid::DrawGrid (void)

{
	long 	hExtent, vExtent;
	short	coord, length, labelVal;
	Str15	str;
	
	GetPixelExtent( &hExtent, &vExtent);

	/* horizontal grid lines */
	length = hExtent - 1;
	for (coord = boxHeight - 1; coord < vExtent; coord += boxHeight) {
		MoveTo(0, coord);
		LineTo(length, coord);
	}
	
	/* vertical grid lines */
	length = vExtent - 1;
	for (coord = boxWidth - 1; coord < hExtent; coord += boxWidth) {
		MoveTo(coord, boxHeight - 1);  /* skip top row */
		LineTo(coord, length);
	}
	
	/* grid labels */
	TextFont(1); TextFace(0); TextSize(10);
	labelVal = 0;
	for (coord = 0; coord < hExtent; coord += boxWidth*2) {
		NumToString(labelVal,str);
		MoveTo(coord + ((boxWidth - StringWidth(str)) / 2), boxHeight - 4);
		DrawString(str);
		labelVal += 20;
	}
}



/***
 * DrawItem
 *
 *  Draw the specified grid item.  The item numbering
 *  scheme is as defined for CGridSelector.
 *
 ***/
 
void CSymWndGrid::DrawItem (short theItem, Rect *theBox)
 
{
	short	sym;
	LongPt	symPos, imgFrame, junk;

	
	if (!ValidItem(theItem))
		return;
	
	sym = ItemToSymCode(theItem);
	 
 	if (sym == 0) {								// sym 0 undefined
 		FillRect(theBox, ltGray);
 		return;
 	}
 	
	FillRect(theBox, white);

 	if (!SymEmpty(sym)) {						// draw thumbnail version of
 												// symbol's canonical image 

 		GetCoords(sym, 0, 0, 0, 0, &imgFrame, &junk, TRUE); 	
 		
	 	symPos.h = theBox->left + ((SCELL_WIDTH - imgFrame.h)  / 2);
	 	symPos.v = theBox->top + ((SCELL_HEIGHT - imgFrame.v) / 2);
	 	
 		DrawSymbol(sym, 0, 0, 0, 0, &thePort->portBits, &symPos, TRUE, FALSE);	
 					
	}
}




/**********************************  Mouse  ***********************************/


/***
 * DoClick (override default method)
 *
 *  Filter out clicks on label cells and symbol 0.
 *
 ***/
 
void CSymWndGrid::DoClick (Point hitPt, short modifierKeys, long when)

{
	short item = FindItem(hitPt);
	
	if (!ValidItem(item) || ItemToSymCode(item) == 0)
		return;
		
	inherited::DoClick(hitPt, modifierKeys, when);
}
